'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.t = t;
exports.jt = jt;
exports.msgid = msgid;
exports.gettext = gettext;
exports.ngettext = ngettext;
exports.addLocale = addLocale;
exports.useLocale = useLocale;
exports.setDedent = setDedent;
exports.useLocales = useLocales;
exports.setDefaultLang = setDefaultLang;
exports.c = c;

var _utils = require('./utils');

var _validation = require('./validation');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var conf = new _config2.default();

function Context(context) {
    if (_utils.isDebug) {
        if (typeof context !== 'string') {
            throw new Error('String type is expected as a first ' + 'argument to c() function.');
        }
    }
    this.getContext = function () {
        return context;
    };
}

var getTransContext = function getTransContext(obj) {
    if (obj instanceof Context) {
        return obj.getContext();
    }
    return '';
};

function isFuzzy(translationObj) {
    return translationObj && translationObj.comments && translationObj.comments.flag === 'fuzzy';
}

function findTransObj(locale, str, ctx) {
    var locales = conf.getAvailLocales();
    var translations = locales[locale] && (locales[locale].translations[ctx] || locales[locale].translations['']);
    var translation = translations && translations[str];
    if (translation && !isFuzzy(translation)) {
        return translation;
    }
    return null;
}

function findTranslation(str, ctx) {
    var locales = conf.getCurrentLocales();
    if (locales.length) {
        for (var i = 0; i < locales.length; i++) {
            var translation = findTransObj(locales[i], str, ctx);
            if (translation) {
                conf.setCurrentLocale(locales[i]);
                return translation;
            }
        }
    }
    return findTransObj(conf.getCurrentLocale(), str, ctx);
}

var maybeDedent = function maybeDedent(str) {
    return conf.isDedent() ? (0, _utils.dedentStr)(str) : str;
};

function t(strings) {
    var result = strings;
    if (strings && strings.reduce) {
        for (var _len = arguments.length, exprs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            exprs[_key - 1] = arguments[_key];
        }

        var id = maybeDedent((0, _utils.getMsgid)(strings, exprs));
        var context = getTransContext(this);
        var transObj = findTranslation(id, context);
        result = transObj ? (0, _utils.msgid2Orig)(transObj.msgstr[0], exprs) : (0, _utils.buildStr)(strings, exprs);
    }
    return maybeDedent(result);
}

var separator = /(\${\s*\d+\s*})/g;
var slotIdRegexp = /\${\s*(\d+)\s*}/;

function jt(strings) {
    for (var _len2 = arguments.length, exprs = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        exprs[_key2 - 1] = arguments[_key2];
    }

    if (strings && strings.reduce) {
        var id = maybeDedent((0, _utils.getMsgid)(strings, exprs));
        var context = getTransContext(this);
        var transObj = findTranslation(id, context);
        if (!transObj) return (0, _utils.buildArr)(strings, exprs);

        // splits string & capturing group into tokens
        //
        var translatedTokens = transObj.msgstr[0].split(separator);

        return translatedTokens.map(function (token) {
            var slotIdMatch = token.match(slotIdRegexp);
            // slotIdMatch is not null only when the token is a variable slot (${xx})
            return slotIdMatch ? exprs[+slotIdMatch[1]] : token;
        });
    }
    return strings;
}

function msgid(strings) {
    /* eslint-disable no-new-wrappers */
    if (strings && strings.reduce) {
        for (var _len3 = arguments.length, exprs = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
            exprs[_key3 - 1] = arguments[_key3];
        }

        var result = new String((0, _utils.buildStr)(strings, exprs));
        result._strs = strings;
        result._exprs = exprs;
        return result;
    }

    return strings;
}

function gettext(id) {
    var context = getTransContext(this);
    var transObj = findTranslation(id, context);
    return transObj ? transObj.msgstr[0] : id;
}

function ngettext() {
    for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
    }

    if (_utils.isDebug) (0, _validation.validateNgettextMsgid)(args[0]);

    var id = maybeDedent((0, _utils.getMsgid)(args[0]._strs, args[0]._exprs));
    var n = args[args.length - 1];
    if (_utils.isDebug) (0, _validation.validateNgettextNumber)(n);

    var forms = args.slice(1, -1);
    forms.unshift(args[0].toString());
    if (_utils.isDebug) (0, _validation.validateNgettextPluralForms)(conf.getDefaultPluralFormsCount(), forms.length);

    var trans = findTranslation(id, getTransContext(this));
    if (trans) {
        var _pluralFn = (0, _utils.getPluralFnForTrans)(trans, conf);
        return maybeDedent((0, _utils.msgid2Orig)(_pluralFn(n, trans.msgstr), args[0]._exprs));
    }
    var pluralFn = conf.getDefaultPluralFn();
    return maybeDedent(pluralFn(n, forms));
}

function addLocale(locale, data) {
    conf.addLocale(locale, data);
}

function useLocale(locale) {
    conf.setCurrentLocale(locale);
}

function setDedent(value) {
    conf.setDedent(Boolean(value));
}

function useLocales(locales) {
    conf.setCurrentLocales(locales);
}

function setDefaultLang(lang) {
    if (_utils.isDebug) (0, _validation.validateLang)(lang);
    conf.setDefaultLang(lang);
}

function c(context) {
    var ctx = new Context(context);
    return {
        t: t.bind(ctx),
        jt: jt.bind(ctx),
        gettext: gettext.bind(ctx),
        ngettext: ngettext.bind(ctx)
    };
}